/*
 * eth_ecat_pdo_fmmu.cpp
 *
 *  Created on: May 16, 2023
 *      Author: algin
 */

#include "ethernet_industry/CoE/eth_ecat_pdo_fmmu.hpp"

namespace free_rtos {

namespace ecat_pdo_fmmu {

void EthEcatPdoFMMU::init() {
    std::vector<ecat_buffer::EcatBufferSlave>& buffer_slaves = ecat_buffer_.get_buffer_slaves();

    pdo_fmmu_slaves_.reserve(buffer_slaves.size());

    for(ecat_buffer::EcatBufferSlave& buffer_slave : buffer_slaves) {
        pdo_fmmu_slaves_.push_back(EcatPdoFMMUSlave{buffer_slave});
    }
}

void EthEcatPdoFMMU::wait_op() {
    free_rtos::Semaphore& init_sem = ecat_buffer_.get_ecat().get_init_sem();
    free_rtos::Semaphore& process_sem = ecat_buffer_.get_ecat().get_process_sem();
    std::array<std::array<uint8_t, 55>, 2> process_data;

    process_sem.pend();

    process_data[0].fill(0x00);
    process_data[1].fill(0x00);

    read(process_data[0], process_data[1]);
/*
    for(uint8_t& byte : process_data[0]) {
        DebugP_log("0x%01x", byte);
    }
    DebugP_log("\r\n");
*/
    write(process_data[0], process_data[1]);

    init_sem.post();
    process_sem.pend();

    for(uint32_t i = 0; i < 250; i++) {
        read(process_data[0], process_data[1]);
/*
        for(uint8_t& byte : process_data[0]) {
            DebugP_log("0x%01x", byte);
        }
        DebugP_log("\r\n");
*/
        write(process_data[0], process_data[1]);

        //ClockP_usleep(125ul);
    }

    init_sem.post();
    process_sem.pend();
}

void EthEcatPdoFMMU::process() {
    std::array<std::array<uint8_t, 55>, 2> process_data;

    wait_op();

    while(1)
    {
        read(process_data[0], process_data[1]);
/*
        for(uint8_t& byte : process_data[0]) {
            DebugP_log("0x%01x", byte);
        }
        DebugP_log("\r\n");
*/
        write(process_data[0], process_data[1]);

        //ClockP_usleep(125ul);
    }
}

}

}
